/*
!xMeter_EA Major_5 Indy v02.mq4
Copyright  2007, MetaQuotes Software Corp.
Price Meter System GPL

This is an indicator based on the EA called !xMeter_EA.mq4 by Robert Hill (5/16/2007)

traderak20@gmail.com
   1/30/2008   Changed currencies to major 5 (EUR, USD, GBP, CHF, JPY). The meter uses all 10 possible pairs
   1/30/2008   Replaced GetPoint() function with built-in function MarketInfo(string symbol,MODE_POINT)
   1/30/2008   Default symbol names are now without "m", for IBFX accounts the "m" is added afterwards
   1/30/2008   Changed the display of grades from a scale of 5 steps to 10 (10 were already calculated anyway)
   1/30/2008   Some minor changes to variable names, clean-up of code and cosmetic.
   1/30/2008   Renamed file from !xMeter_EA.mq4 to !xMeter_EA Major_5 Indy v02.mq4
*/

/*
traderak20@gmail.com

This indicator can be used in addition to the expert advisor created by Robert Hill. The EA provides better
real-time information since it's updating continuously and doesn't need a tick on the current chart as input.
This indicator can then be used to gain more insight in the history of the strength of currencies.

The indicator may slow down your computer for a bit when set to view the entire history of your chart. To allow you
look at only certain parts of the history, I have added three parameters: setStartDateTime, setEndDateTime, maxHistoryBars
These parameters can be set separately or combined in a number of ways:
   * use only maxHistoryBars, leave start date and end date empty:                        ---------------H++++0
   * set start date + end date (maxHistoryBars is ignored):                               ----S+++++++E-------0
   * use start date + maxHistoryBars, leave end date empty:                               ----S++++H----------0
   * set maxHistoryBars + end date, leave start date empty:                               -------H++++E-------0
   * set only a start date, leave end date and maxHistoryBars empty:                      ----S+++++++++++++++0
   * set only an end date, leave start date and maxHistoryBars empty:                     ++++++++++++E-------0
   * leave the start date, end date and maxHistoryBars empty to view the whole history:   ++++++++++++++++++++0
*/

#property copyright "x Meter System GPL"
#property link      "forex-tsd dot com"

#property indicator_separate_window
#property indicator_minimum 0
#property indicator_maximum 11
#property indicator_buffers 5
#property indicator_color1 DodgerBlue
#property indicator_color2 ForestGreen
#property indicator_color3 FireBrick
#property indicator_color4 LightBlue
#property indicator_color5 Orange

//---- input parameters
extern bool showAllCurrencies = TRUE;      // TRUE = show all five indicator lines, FALSE = show only the currencies from this chart
extern bool AccountIsIBFXmini = FALSE;
extern string setStartDateTime = "";       // Use standard date format. 2007.5.25  Optionally add time as well: 2007.5.25 16:00
extern string setEndDateTime = "";         // Use standard date format. 2007.7.31  Optionally add time as well: 2007.5.25 16:00
extern int maxHistoryBars = 500;           // This value is ignored if both a start and end datetime are given
extern bool lookBackStartMidnight = TRUE;  // TRUE = calculate with high, low and range since midnight
                                           // FALSE = calculate with high, low and range from rolling lookback period (in hours)
extern int lookBackPeriod = 12;
extern bool applySmoothing = FALSE;        // Apply smoothing to get a less choppy picture (with more lag obviously)
extern int smoothingPeriod = 14;

//---- buffers
double EUR_Buffer[];
double USD_Buffer[];
double GBP_Buffer[];
double CHF_Buffer[];
double JPY_Buffer[];

//---- Number of pairs
#define ARRSIZE 10
//---- Number of currencies
#define PAIRSIZE 5
//---- Scale of currency's power
#define TABSIZE 10

//---- Set constant for currency pairs; added to make code easier to understand
#define CHFJPY 0
#define EURCHF 1
#define EURGBP 2
#define EURJPY 3
#define EURUSD 4
#define GBPCHF 5
#define GBPJPY 6
#define GBPUSD 7
#define USDCHF 8
#define USDJPY 9

//---- Set constant for currencies; added to make code easier to understand
#define EUR 0
#define USD 1
#define GBP 2
#define CHF 3
#define JPY 4

string aPair[ARRSIZE]   = {"CHFJPY","EURCHF","EURGBP","EURJPY","EURUSD",
                           "GBPCHF","GBPJPY","GBPUSD","USDCHF","USDJPY"};
string aMajor[PAIRSIZE] = {"EUR","USD","GBP","CHF","JPY"};

//+------------------------------------------------------------------+
//     expert initialization function                                |       
//+------------------------------------------------------------------+
int init()
{
   IndicatorBuffers(5);

   IndicatorDigits(MarketInfo(Symbol(),MODE_DIGITS));

//---- indicator line
   SetIndexBuffer(0,EUR_Buffer);
   SetIndexBuffer(1,USD_Buffer);
   SetIndexBuffer(2,GBP_Buffer);
   SetIndexBuffer(3,CHF_Buffer);
   SetIndexBuffer(4,JPY_Buffer);

   bool showEUR, showUSD, showGBP, showCHF, showJPY;
   if ( StringFind(Symbol(), "EUR", 0) != -1) showEUR = TRUE;
   if ( StringFind(Symbol(), "USD", 0) != -1) showUSD = TRUE;
   if ( StringFind(Symbol(), "GBP", 0) != -1) showGBP = TRUE;
   if ( StringFind(Symbol(), "CHF", 0) != -1) showCHF = TRUE;
   if ( StringFind(Symbol(), "JPY", 0) != -1) showJPY = TRUE;

   if ( showAllCurrencies == TRUE || showEUR == TRUE ) SetIndexStyle(0,DRAW_LINE,STYLE_SOLID,1,DodgerBlue);
   else SetIndexStyle(0,DRAW_LINE,STYLE_SOLID,0,CLR_NONE);
   if ( showAllCurrencies == TRUE || showUSD == TRUE ) SetIndexStyle(1,DRAW_LINE,STYLE_SOLID,1,ForestGreen);
   else SetIndexStyle(1,DRAW_LINE,STYLE_SOLID,0,CLR_NONE);
   if ( showAllCurrencies == TRUE || showGBP == TRUE ) SetIndexStyle(2,DRAW_LINE,STYLE_SOLID,1,FireBrick);
   else SetIndexStyle(2,DRAW_LINE,STYLE_SOLID,0,CLR_NONE);
   if ( showAllCurrencies == TRUE || showCHF == TRUE ) SetIndexStyle(3,DRAW_LINE,STYLE_SOLID,1,LightBlue);
   else SetIndexStyle(3,DRAW_LINE,STYLE_SOLID,0,CLR_NONE);
   if ( showAllCurrencies == TRUE || showJPY == TRUE ) SetIndexStyle(4,DRAW_LINE,STYLE_SOLID,1,Orange);
   else SetIndexStyle(4,DRAW_LINE,STYLE_SOLID,0,CLR_NONE);

   SetIndexLabel(0,"EUR");
   SetIndexLabel(1,"USD");
   SetIndexLabel(2,"GBP");
   SetIndexLabel(3,"CHF");
   SetIndexLabel(4,"JPY");

   SetIndexDrawBegin(0,0);
   SetIndexDrawBegin(1,0);
   SetIndexDrawBegin(2,0);
   SetIndexDrawBegin(3,0);
   SetIndexDrawBegin(4,0);

//---- If you change the name of this window,
//---- you must also update the WindowFind() function further down
   IndicatorShortName("Price Meter System GPL -");
//----
   return(0);
}
//+------------------------------------------------------------------+
//     expert deinitialization function                              |       
//+------------------------------------------------------------------+
int deinit()
{
   deleteLegendObjects();

   return(0);
}
//+------------------------------------------------------------------+
//     expert start function                                         |       
//+------------------------------------------------------------------+
int start()
{
//---- Date, time and counter variables
   int counted_bars = IndicatorCounted();
   int timeFrame_1 = Period();
   int barShift_TF1, startBar_TF1, endBar_TF1;

//---- Minimum 100 bars in chart
   if ( iBars(Symbol(), timeFrame_1) <= 100 ) return(0);
//---- Check for possible errors
   if ( counted_bars < 0 ) return(-1);
//---- The last counted bar will be recounted
   if( counted_bars > 0 ) counted_bars--;

//---- Set start/end bar
   setStartEndBar(startBar_TF1, endBar_TF1);

//---- Price Meter System variables
   double aMeter[PAIRSIZE];
   double aHigh[ARRSIZE];
   double aLow[ARRSIZE];
   double aClose[ARRSIZE];
   double aRatio_1[ARRSIZE];
   double aRatio_2[ARRSIZE];
   double aRange[ARRSIZE];
   double aLookup_1[ARRSIZE];
   double aLookup_2[ARRSIZE];
   double point;
   int i;
   string mySymbol;
   int highBar, lowBar;

//---- Main calculation loop
   for ( barShift_TF1 = startBar_TF1; barShift_TF1 >= endBar_TF1; barShift_TF1-- )
   {
//---- Set lookback period to start at midnight
      if ( lookBackStartMidnight == TRUE )
         lookBackPeriod = TimeHour( iTime(mySymbol, timeFrame_1, barShift_TF1) ) + 1;

//---- Initialize all pairs required value
      for ( i = 0; i < ARRSIZE; i++ )
      {
//---- Make naming compatible with IBFX mini accounts
         if (AccountIsIBFXmini) mySymbol = aPair[i] + "m";
         else mySymbol = aPair[i];
//---- Get point info for symbol
         point = MarketInfo(mySymbol,MODE_POINT);
//---- Find highest and lowest bar during the lookback period
         highBar = iHighest(mySymbol, timeFrame_1, MODE_HIGH, lookBackPeriod, barShift_TF1);
         lowBar = iLowest(mySymbol, timeFrame_1, MODE_LOW, lookBackPeriod, barShift_TF1);
//---- Set high, low and close
         aHigh[i] = iHigh(mySymbol, timeFrame_1, highBar);
         aLow[i] = iLow(mySymbol, timeFrame_1, lowBar);
         aClose[i] = iClose(mySymbol, timeFrame_1, barShift_TF1);
//---- Calculate today's range
         aRange[i] = MathMax( (aHigh[i] - aLow[i]) / point,1);
//---- Calculate pair's ratio and inverse ratio
         aRatio_1[i] = ( aClose[i] - aLow[i]) / aRange[i] / point;
         aRatio_2[i] = ( aHigh[i] - aClose[i]) / aRange[i] / point;
//---- Set a pair's grade and inverse grade
         aLookup_1[i] = iLookup( aRatio_1[i] * 100 );
         aLookup_2[i] = iLookup( aRatio_2[i] * 100 );
      }

//---- Calculate and normalize all values to display in graph
      aMeter[EUR] = NormalizeDouble( (aLookup_1[EURCHF] + aLookup_1[EURGBP] + aLookup_1[EURJPY] + aLookup_1[EURUSD] )/4,1);
      aMeter[USD] = NormalizeDouble( (aLookup_2[EURUSD] + aLookup_2[GBPUSD] + aLookup_1[USDJPY] + aLookup_1[USDCHF] )/4,1);
      aMeter[GBP] = NormalizeDouble( (aLookup_2[EURGBP] + aLookup_1[GBPCHF] + aLookup_1[GBPJPY] + aLookup_1[GBPUSD] )/4,1);
      aMeter[CHF] = NormalizeDouble( (aLookup_1[CHFJPY] + aLookup_2[EURCHF] + aLookup_2[GBPCHF] + aLookup_2[USDCHF] )/4,1);
      aMeter[JPY] = NormalizeDouble( (aLookup_2[CHFJPY] + aLookup_2[EURJPY] + aLookup_2[GBPJPY] + aLookup_2[USDJPY] )/4,1);

//---- Show values without smoothing
      if (barShift_TF1 == startBar_TF1 || applySmoothing == FALSE)
      {
         EUR_Buffer[barShift_TF1] = aMeter[EUR];
         USD_Buffer[barShift_TF1] = aMeter[USD];
         GBP_Buffer[barShift_TF1] = aMeter[GBP];
         CHF_Buffer[barShift_TF1] = aMeter[CHF];
         JPY_Buffer[barShift_TF1] = aMeter[JPY];
      }
//---- Show values with smoothing
      else
      {
         EUR_Buffer[barShift_TF1] = (EUR_Buffer[barShift_TF1 + 1] * (smoothingPeriod-1) + aMeter[EUR]) / smoothingPeriod;
         USD_Buffer[barShift_TF1] = (USD_Buffer[barShift_TF1 + 1] * (smoothingPeriod-1) + aMeter[USD]) / smoothingPeriod;
         GBP_Buffer[barShift_TF1] = (GBP_Buffer[barShift_TF1 + 1] * (smoothingPeriod-1) + aMeter[GBP]) / smoothingPeriod;
         CHF_Buffer[barShift_TF1] = (CHF_Buffer[barShift_TF1 + 1] * (smoothingPeriod-1) + aMeter[CHF]) / smoothingPeriod;
         JPY_Buffer[barShift_TF1] = (JPY_Buffer[barShift_TF1 + 1] * (smoothingPeriod-1) + aMeter[JPY]) / smoothingPeriod;
      }
   }

   deleteLegendObjects();
   createLegendObjects("objTextEUR",300,2,"EUR",10,"Arial",DodgerBlue);
   createLegendObjects("objTextUSD",335,2,"USD",10,"Arial",ForestGreen);
   createLegendObjects("objTextGBP",370,2,"GBP",10,"Arial",FireBrick);
   createLegendObjects("objTextCHF",405,2,"CHF",10,"Arial",LightBlue);
   createLegendObjects("objTextJPY",440,2,"JPY",10,"Arial",Orange);

   return(0);
}

//---- This function will return a grade value for a pair based on its power
int iLookup(double ratio)
{
//---- Grade table for currency's power
   int aTable[TABSIZE]  = {0,3,10,25,40,50,60,75,90,97};
   int index;

   if (ratio <= aTable[0]) index = 0;
   else if (ratio < aTable[1]) index = 0;
   else if (ratio < aTable[2]) index = 1;
   else if (ratio < aTable[3]) index = 2;
   else if (ratio < aTable[4]) index = 3;
   else if (ratio < aTable[5]) index = 4;
   else if (ratio < aTable[6]) index = 5;
   else if (ratio < aTable[7]) index = 6;
   else if (ratio < aTable[8]) index = 7;
   else if (ratio < aTable[9]) index = 8;
   else index = 9;

   return(index);
}

//---- Create the objects that serve as legend for the indicator buffers
void createLegendObjects(string name, int xcoor, int ycoor, string text, int fsize, string ftype, color fcolor)
{
//---- Variable used to determine this window's index. Required for painting legend objects in correct window
   int wIndex = WindowFind("Price Meter System GPL -");
   
   ObjectCreate(name,OBJ_LABEL,wIndex,0,0);
   ObjectSet(name,OBJPROP_CORNER,0);
   ObjectSet(name,OBJPROP_XDISTANCE,xcoor);
   ObjectSet(name,OBJPROP_YDISTANCE,ycoor);
   ObjectSetText(name,text,fsize,ftype,fcolor);

   return(0);
}

void deleteLegendObjects()
{
   ObjectDelete("objTextEUR");
   ObjectDelete("objTextUSD");
   ObjectDelete("objTextGBP");
   ObjectDelete("objTextCHF");
   ObjectDelete("objTextJPY");

   return(0);
}

//---- Set the start/end bar for the indicator based on setStartDateTime, setEndDateTime and maxHistoryBars
void setStartEndBar(int& startbar, int& endbar)
{
   int maxbars = iBars(Symbol(), Period()) - IndicatorCounted() - 1;
   
   if ( setStartDateTime != "" && setEndDateTime != "" )
   {
      if ( StrToTime(setStartDateTime) > StrToTime(setEndDateTime) )
      {
         string tempTimeString = setStartDateTime;
         setStartDateTime = setEndDateTime;
         setEndDateTime = tempTimeString;
      }
      startbar = iBarShift(Symbol(), Period(), StrToTime(setStartDateTime), false);
      endbar = iBarShift(Symbol(), Period(), StrToTime(setEndDateTime), false);
   }

   if ( setStartDateTime != "" && setEndDateTime == "" && maxHistoryBars > 0)
   {
      startbar = iBarShift(Symbol(), Period(), StrToTime(setStartDateTime), false);
//---- Don't let endbar be smaller than most recent bar
      endbar = MathMax(startbar - maxHistoryBars,0);
   }
   if ( setStartDateTime != "" && setEndDateTime == "" && maxHistoryBars <= 0)
   {
      startbar = iBarShift(Symbol(), Period(), StrToTime(setStartDateTime), false);
      endbar = 0;
   }

   if ( setStartDateTime == "" && setEndDateTime != "" && maxHistoryBars > 0)
   {
      endbar = iBarShift(Symbol(), Period(), StrToTime(setEndDateTime), false);
//---- Don't go let startbar be bigger than the maximum bars available in this chart
      startbar = MathMin( endbar + maxHistoryBars, maxbars );
   }
   if ( setStartDateTime == "" && setEndDateTime != "" && maxHistoryBars <= 0)
   {
      startbar = maxbars;
      endbar = iBarShift(Symbol(), Period(), StrToTime(setEndDateTime), false);
   }
   
   if ( setStartDateTime == "" && setEndDateTime == "" && maxHistoryBars > 0)
   {
      startbar = maxHistoryBars;
      endbar = 0;
   }
   if ( setStartDateTime == "" && setEndDateTime == "" && maxHistoryBars <= 0)
   {
      startbar = maxbars;
      endbar = 0;  
   }
   
   return(0);
}
//+------------------------------------------------------------------+

